@@ -18,8 +18,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 /*
   @NOTATION@
  */
@@ -36,7 +36,6 @@
 #include <sys/stat.h>
 
 #include <glib.h>
-#include <glib/goption.h>
 #include <glib/gstdio.h>
 
 #include "gnome-i18n.h"
