@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with the Gnome Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 /*
   @NOTATION@
@@ -30,8 +30,12 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <sys/types.h>
 #include <unistd.h>
+#include <fcntl.h>
+#include <errno.h>
 #include <time.h>
+#include <glib.h>
 
 #ifdef HAVE_ESD
 #include <esd.h>
@@ -413,6 +417,67 @@ gnome_sound_sample_load_audiofile(const char *file)
 }
 #endif
 
+#ifdef HAVE_ESD
+static int
+send_all (int fd, const char *buf, size_t buflen)
+{
+	GPollFD pfd[1];
+	size_t nwritten = 0;
+	int flags, rv;
+	ssize_t n;
+	
+	if ((flags = fcntl (fd, F_GETFL)) == -1)
+		return -1;
+	
+	fcntl (fd, F_SETFL, flags | O_NONBLOCK);
+	
+	pfd[0].events = G_IO_OUT;
+	pfd[0].fd = fd;
+	
+	do {
+		do {
+			pfd[0].revents = 0;
+			rv = g_poll (pfd, 1, 100);
+		} while (rv == -1 && (errno == EINTR || errno == EAGAIN));
+		
+		if (rv == -1) {
+			if (errno != EBADF) {
+				rv = errno;
+				esd_close (fd);
+				errno = rv;
+			}
+			
+			fd = -1;
+			break;
+		} else if (rv < 1 || (pfd[0].revents & (G_IO_ERR | G_IO_HUP | G_IO_OUT)) != G_IO_OUT) {
+			/* we /just/ lost the esd connection */
+			esd_close (fd);
+			fd = -1;
+			break;
+		} else {
+			/* socket is ready for writing */
+			do {
+				n = write (fd, buf + nwritten, buflen - nwritten);
+			} while (n == -1 && errno == EINTR);
+			
+			if (n == -1) {
+				rv = errno;
+				esd_close (fd);
+				errno = rv;
+				fd = -1;
+				break;
+			}
+			
+			nwritten += n;
+		}
+	} while (nwritten < buflen);
+	
+	if (fd != -1 && flags != -1)
+		fcntl (fd, F_SETFL, flags);
+	
+	return fd;
+}
+#endif
 
 /**
  * gnome_sound_sample_load:
@@ -469,21 +534,23 @@ gnome_sound_sample_load(const char *sample_name, const char *filename)
 	   * file, or event type, for later identification */
 	  s->id = esd_sample_cache (gnome_sound_connection, s->format, s->rate,
 				    size, (char *)sample_name);
-	  write (gnome_sound_connection, s->data, size);
-	  confirm = esd_confirm_sample_cache (gnome_sound_connection);
+	  
+	  gnome_sound_connection = send_all (gnome_sound_connection, (const char *) s->data, size);
+	  if (gnome_sound_connection != -1)
+	    confirm = esd_confirm_sample_cache (gnome_sound_connection);
+	  
 	  if (s->id <= 0 || confirm != s->id)
 	    {
 	      g_warning ("error caching sample <%d>!\n", s->id);
 	      s->id = 0;
 	    }
-	  g_free (s->data);
-	  s->data = NULL;
 	}
     }
 
   sample_id = s->id;
 
-  g_free(s->data); g_free(s);
+  g_free(s->data);
+  g_free(s);
 
   return sample_id;
 #else
@@ -521,9 +588,12 @@ gnome_sound_play (const char * filename)
 
   sample = gnome_sound_sample_load (buf, filename);
 
-  esd_sample_play(gnome_sound_connection, sample);
-  fsync (gnome_sound_connection);
-  esd_sample_free(gnome_sound_connection, sample);
+  if (gnome_sound_connection != -1 && sample > 0)
+    {
+      esd_sample_play(gnome_sound_connection, sample);
+      fsync (gnome_sound_connection);
+      esd_sample_free(gnome_sound_connection, sample);
+    }
 #endif
 }
 
